/* Dynamic Array Library
 *
 * Author: Alptekin Kp
 * Implementation Date: 07.2003
 * Contact address: alpkup@programmer.net
 * 
 * These files and codes are fully distributable if no change is made on them.
 */

// Includes
#include "DynamicArray.h"


// Creates a new array with default empty content and 0 size
// Maximum size of the array is defined in "DynamicArray.h"
DynamicArray* newDynamicArray(void)
{
	DynamicArray* a = (DynamicArray*)malloc(sizeof(DynamicArray));
	a->array = (void**)calloc(MAX_HASH , sizeof(void*));
	a->next = NULL;
	a->currentSize = 0;
	return a;
}

// Adds the given element to the first available place
// Starting from the given array (head of the list)
// Creates a new array if current arrays are all full
// Returns FALSE if given head array is NULL
boolean addElement(DynamicArray* head , void* element)
{
	DynamicArray* temp;

	if (head == NULL)
		return FALSE;

	temp = head;
	while (temp->currentSize >= MAX_HASH)
	{
		if (temp->next != NULL)
		{
			temp = temp->next;
		}
		else
		{
			temp->next = newDynamicArray();
			temp = temp->next;
		}
	}

	temp->array[temp->currentSize] = element;
	temp->currentSize++;
	return TRUE;
}

// Finds and returns the element with the given element no
// Looking into all arrays in the list whose head is given
// Returns NULL if given head array is NULL
void* getElementAt(DynamicArray* head , int elementNo)
{
	DynamicArray* temp;

	if (head == NULL)
		return NULL;

	temp = head;
	while (elementNo >= MAX_HASH)
	{
		elementNo -= MAX_HASH;
		temp = temp->next;
	}
	
	return temp->array[elementNo];
}

// Removes the last element in the array
// Returns FALSE if given head array is NULL
boolean removeElement(DynamicArray* head)
{
	DynamicArray* temp;

	if (head == NULL)
		return FALSE;

	temp = head;
	while (temp->next != NULL)
	{
		temp = temp->next;
	}
	
	temp->array[temp->currentSize] = NULL;
	temp->currentSize--;

	return TRUE;
}

// Returns the total number of elements in the list starting with head
// Returns -1 if given head array is NULL
int getElementCount(DynamicArray* head)
{
	DynamicArray* temp;
	int totalSize;

	if (head == NULL)
		return -1;

	temp = head;
	totalSize = 0;
	while (temp != NULL)
	{
		totalSize += temp->currentSize;
		temp = temp->next;
	}

	return totalSize;
}

// Returns the length of the linked list of dynamic arrays
// Returns -1 if given head array is NULL
// If this function returns a high value, try to increase MAX_HASH value
// defined in "DynamicArray.h" to increase performance
int getArrayListLength(DynamicArray* head)
{
	DynamicArray* temp;
	int length;

	if (head == NULL)
		return -1;

	temp = head;
	length = 0;
	while (temp != NULL)
	{
		length++;
		temp = temp->next;
	}

	return length;
}

/*******************************************************************/
// Additional functions

/* Important Notice:
 * Do not use following functions if not absolutely necessary
 * Or be sure to use properly (with proper parameters)
 * Otherwise some anormalities may occur
 */

// Adds an element to the given position
// Creates new arrays if necessary
// Returns FALSE if given head array is NULL
// Returns FALSE if overwrite is FALSE and there exists an item at given position
// Does not count for current size (element count)
boolean addElementAt(DynamicArray* head , void* element , int position , boolean overwrite)
{
	DynamicArray* temp;

	if (head == NULL)
		return FALSE;

	temp = head;
	while (position >= MAX_HASH)
	{
		position -= MAX_HASH;
		if (temp->next != NULL)
		{
			temp = temp->next;
		}
		else
		{
			temp->next = newDynamicArray();
			temp = temp->next;
		}
	}

	if (overwrite == FALSE && temp->array[position] != NULL)
		return FALSE;

	temp->array[position] = element;

	return TRUE;
}

// Removes the element at the given position
// Returns FALSE if given head array is NULL
// Be sure not to give a bad position in order not to get 
// "ACCESS VIOLATION" error
boolean removeElementAt(DynamicArray* head , int position)
{
	DynamicArray* temp;

	if (head == NULL)
		return FALSE;

	temp = head;
	while (position >= MAX_HASH)
	{
		position -= MAX_HASH;
		temp = temp->next;
	}
	
	temp->array[position] = NULL;

	return TRUE;
}
